#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=92:

"""
Holds the ExtractFirstOldRAR class.

This module works with old-style rar sets which need to be extracted
before they are verified.

It will detect sets like the following:
X.par2
X.vol0+1.par2
...

X.rar
X.r00
X.r01
...

Where the PAR2 files do not protect the rar files directly, but instead
protect the files contained inside the rar files.
"""

__author__    = "Ira W. Snyder (devel@irasnyder.com)"
__copyright__ = "Copyright (c) 2006,2007 Ira W. Snyder (devel@irasnyder.com)"
__license__   = "GNU GPL v2 (or, at your option, any later version)"

#    ExtractFirstOldRAR.py -- detect and work with a old-style rar set
#
#    Copyright (C) 2006,2007  Ira W. Snyder (devel@irasnyder.com)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from PAR2Set import ExtractFirstBase, OldRAR, utils

class ExtractFirstOldRAR(ExtractFirstBase, OldRAR):

    def detect(self):
        regex = r'^.*\.r00$'
        m1 = utils.hasAMatch(regex, self.similarlyNamedFiles)
        m2 = utils.hasAMatch(regex, self.protectedFiles)

        if m1 and not m2:
            return

        raise TypeError

