#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=92:

"""
Holds the Join class.

This module works with normal joined sets.

It will detect sets like the following:
X.par2
X.vol0+1.par2
...

X.001
X.002
...

Where the PAR2 files protect a file named X.avi, or similar other types.
It will not work where the PAR2 files are protecting the X.001, etc files
directly.
"""

__author__    = "Ira W. Snyder (devel@irasnyder.com)"
__copyright__ = "Copyright (c) 2006,2007 Ira W. Snyder (devel@irasnyder.com)"
__license__   = "GNU GPL v2 (or, at your option, any later version)"

#    Join.py
#
#    Copyright (C) 2006,2007  Ira W. Snyder (devel@irasnyder.com)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, re, logging
from PAR2Set import Base, utils

class Join(Base):

    ############################################################################

    def detect(self):

        regex = r'^.*\.\d\d\d$'
        m1 = utils.hasAMatch(regex, self.similarlyNamedFiles)
        m2 = utils.hasAMatch(regex, self.protectedFiles)

        # This is a good match if this criteria is met
        if m1 and not m2:
            return

        raise TypeError

    ############################################################################

    def repair(self):

        regex = r'^.*\.\d\d\d$'
        files = utils.findMatches(regex, self.similarlyNamedFiles)
        utils.runCommand(['par2repair'] + self.PAR2Files + files, self.directory)

    ############################################################################

    def findDeletableFiles(self):

        files = Base.findDeletableFiles(self)
        files = [f for f in files if f not in self.protectedFiles]

        return files

    ############################################################################

