#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=80:

# This breaks my convention, but python requires that it be
# at the top of the file ...
from __future__ import with_statement

"""
Module which holds PAR2 file parsing functions.

Much of this code was borrowed from the excellent cfv project.
See http://cfv.sourceforge.net/ for a copy.
"""

__author__    = "Ira W. Snyder (devel@irasnyder.com)"
__copyright__ = "Copyright (c) 2006-2008 Ira W. Snyder (devel@irasnyder.com)"
__license__   = "GNU GPL v2 (or, at your option, any later version)"

#    par2parser.py -- PAR2 file parsing utility
#
#    Copyright (C) 2006-2008  Ira W. Snyder (devel@irasnyder.com)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import struct, errno, os, md5

################################################################################

def chompnulls(line):
    p = line.find('\0')

    if p < 0:
        return line
    else:
        return line[:p]

################################################################################

# Get all of the filenames protected by the parityFile
#
# This code was almost copied from the CFV project
# See the header of this file for more information
def getProtectedFiles(directory, parityFile):

    assert os.path.isdir(directory)
    assert os.path.isfile(os.path.join(directory, parityFile))

    with open(os.path.join(directory, parityFile), 'rb') as file:

        # We always want to do crc checks
        docrcchecks = True

        pkt_header_fmt = '< 8s Q 16s 16s 16s'
        pkt_header_size = struct.calcsize(pkt_header_fmt)
        file_pkt_fmt = '< 16s 16s 16s Q'
        file_pkt_size = struct.calcsize(file_pkt_fmt)
        main_pkt_fmt = '< Q I'
        main_pkt_size = struct.calcsize(main_pkt_fmt)

        seen_file_ids = {}
        expected_file_ids = None
        filenames = []

        while True:
            d = file.read(pkt_header_size)
            if not d:
                break

            magic, pkt_len, pkt_md5, set_id, pkt_type = struct.unpack(pkt_header_fmt, d)

            if docrcchecks:
                control_md5 = md5.new()
                control_md5.update(d[0x20:])
                d = file.read(pkt_len - pkt_header_size)
                control_md5.update(d)

                if control_md5.digest() != pkt_md5:
                    raise EnvironmentError, (errno.EINVAL, \
                        "corrupt par2 file - bad packet hash")

            if pkt_type == 'PAR 2.0\0FileDesc':
                if not docrcchecks:
                    d = file.read(pkt_len - pkt_header_size)

                file_id, file_md5, file_md5_16k, file_size = \
                    struct.unpack(file_pkt_fmt, d[:file_pkt_size])

                if seen_file_ids.get(file_id) is None:
                    seen_file_ids[file_id] = 1
                    filename = chompnulls(d[file_pkt_size:])
                    filenames.append(filename)

            elif pkt_type == "PAR 2.0\0Main\0\0\0\0":
                if not docrcchecks:
                    d = file.read(pkt_len - pkt_header_size)

                if expected_file_ids is None:
                    expected_file_ids = []
                    slice_size, num_files = struct.unpack(main_pkt_fmt, d[:main_pkt_size])
                    num_nonrecovery = (len(d)-main_pkt_size)/16 - num_files

                    for i in range(main_pkt_size,main_pkt_size+(num_files+num_nonrecovery)*16,16):
                        expected_file_ids.append(d[i:i+16])

            else:
                if not docrcchecks:
                    file.seek(pkt_len - pkt_header_size, 1)

        if expected_file_ids is None:
            raise EnvironmentError, (errno.EINVAL, \
                "corrupt or unsupported par2 file - no main packet found")

        for id in expected_file_ids:
            if not seen_file_ids.has_key(id):
                raise EnvironmentError, (errno.EINVAL, \
                    "corrupt or unsupported par2 file - " \
                    "expected file description packet not found")

    return filenames

################################################################################

def main():
    pass

################################################################################

if __name__ == '__main__':
    main()

